import math
import unittest
import random

from bridson import poisson_disc_samples


class PoissonDiscSamplingRadiusTest(unittest.TestCase):
    def setUp(self):
        self.prng = random.Random()
        self.prng.seed(42)

    def test_r10(self):
        r = 10
        samples = poisson_disc_samples(100, 100, r=r, random=self.prng.random)
        self.assertGreater(len(samples), 50)
        for i, p in enumerate(samples):
            for q in samples[i + 1:]:
                dx = p[0] - q[0]
                dy = p[1] - q[1]
                self.assertGreater(math.sqrt(dx * dx + dy * dy), r)

    def test_r5(self):
        r = 5
        samples = poisson_disc_samples(100, 100, r=r, random=self.prng.random)
        self.assertGreater(len(samples), 200)
        for i, p in enumerate(samples):
            for q in samples[i + 1:]:
                dx = p[0] - q[0]
                dy = p[1] - q[1]
                self.assertGreater(math.sqrt(dx * dx + dy * dy), r)

    def test_r50(self):
        r = 50
        samples = poisson_disc_samples(100, 100, r=r, random=self.prng.random)
        self.assertGreater(len(samples), 2)
        for i, p in enumerate(samples):
            for q in samples[i + 1:]:
                dx = p[0] - q[0]
                dy = p[1] - q[1]
                self.assertGreater(math.sqrt(dx * dx + dy * dy), r)



from random import random
from math import cos, sin, floor, sqrt, pi, ceil


def euclidean_distance(a, b):
    dx = a[0] - b[0]
    dy = a[1] - b[1]
    return sqrt(dx * dx + dy * dy)


def poisson_disc_samples(width, height, r, k=5, distance=euclidean_distance, random=random):
    tau = 2 * pi
    cellsize = r / sqrt(2)

    grid_width = int(ceil(width / cellsize))
    grid_height = int(ceil(height / cellsize))
    grid = [None] * (grid_width * grid_height)

    def grid_coords(p):
        return int(floor(p[0] / cellsize)), int(floor(p[1] / cellsize))

    def fits(p, gx, gy):
        yrange = list(range(max(gy - 2, 0), min(gy + 3, grid_height)))
        for x in range(max(gx - 2, 0), min(gx + 3, grid_width)):
            for y in yrange:
                g = grid[x + y * grid_width]
                if g is None:
                    continue
                if distance(p, g) <= r:
                    return False
        return True

    p = width * random(), height * random()
    queue = [p]
    grid_x, grid_y = grid_coords(p)
    grid[grid_x + grid_y * grid_width] = p

    while queue:
        qi = int(random() * len(queue))
        qx, qy = queue[qi]
        queue[qi] = queue[-1]
        queue.pop()
        for _ in range(k):
            alpha = tau * random()
            d = r * sqrt(3 * random() + 1)
            px = qx + d * cos(alpha)
            py = qy + d * sin(alpha)
            if not (0 <= px < width and 0 <= py < height):
                continue
            p = (px, py)
            grid_x, grid_y = grid_coords(p)
            if not fits(p, grid_x, grid_y):
                continue
            queue.append(p)
            grid[grid_x + grid_y * grid_width] = p
    return [p for p in grid if p is not None]



from setuptools import setup, find_packages


with open('README.rst') as f:
    description = f.read()


setup(
    name='bridson',
    url='http://github.com/emulbreh/bridson/',
    version='0.1.0',
    packages=find_packages(),
    license=u'BSD License',
    author=u'Johannes Dollinger',
    description=u'poisson disc sampling of 2-dimensional',
    long_description=description,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3'
    ]
)
